// $Id: SpacingDS.cpp,v 1.12 2007/04/27 19:57:41 lpsmith Exp $

/* 
 Copyright 2002 Patrick Colacurcio, Peter Beerli, Mary Kuhner, 
                Jon Yamato and Joseph Felsenstein

 This software is distributed free of charge for non-commercial use
 and is copyrighted.  Of course, we do not guarantee that the software
 works, and are not responsible for any damage you may cause or have.
*/

#include <algorithm>

#include "Converter_SpacingDS.h"
#include "Converter_DataSourceException.h"
#include "stringx.h"

//_____________________________________________________________

SpacingDS::SpacingDS() 
 : DataSourceIf(),
   m_length(0),
   m_offset(1),
   m_map_position(1)
{
} /* SpacingDS::default ctor */

//_____________________________________________________________

void SpacingDS::CheckConsistency(long nmarkers) const
{
  if (m_positions.back() - m_positions.front() >= m_length)
    {
      /*
      string errormsg = "The input length is less than the length ";
      errormsg += "covered by the markers.";
      throw InconsistentDataError(errormsg);
      */
      throw MarkerLengthMismatchDataError(  m_positions.front(),
                                            m_positions.back(),
                                            m_length);
    }

  /*
  if (m_offset > m_positions.front() || 
      m_offset + m_length <= m_positions.back())
    {
      string errormsg = "The given start of the region places some markers ";
      errormsg += "outside of the region.";
      throw InconsistentDataError(errormsg);
    }
    */

  if (m_offset > m_positions.front())
  {
    throw OffsetAfterFirstPositionDataError(m_offset,m_positions.front());
  }

  if (m_offset + m_length <= m_positions.back())
  {
    throw RegionEndBeforeLastPositionDataError(m_offset,m_length,m_positions.back());
  }

  long npositions = static_cast<long>(m_positions.size());

  if (nmarkers != npositions)
    {
      /*
      string errormsg = "There are " + ToString(nmarkers);
      errormsg += " markers in your data and ";
      errormsg += ToString(npositions) + " positions for them to occupy.";

      throw InconsistentDataError(errormsg);
      */
      throw MarkerPositionMismatchDataError(nmarkers,npositions);
    }

} /* SpacingDS::CheckInternalConsistency */

//_____________________________________________________________

SpacingDS::SpacingDS(long length, long nmarkers) : DataSourceIf(), m_length(length),
  m_offset(1), m_map_position(1)
{
/* This constructor makes a SpacingDS when no information except
length is given.  It assumes that all of the markers are lined
up at the left end of the given length.  Data files produced
in this way MUST NOT BE USED TO INFER RECOMBINATION.  The faculty
is provided only for non-recombination uses. */

long i; 
for (i = 0; i < nmarkers; ++i) {
  m_positions.push_back(i);
}

} /* SpacingDS::ctor */

//_____________________________________________________________

SpacingDS::SpacingDS(const vector<long>& positions, long length,
  long nmarkers) :
DataSourceIf(), m_positions(positions), m_length(length)
{
  std::sort(m_positions.begin(), m_positions.end());

  m_offset = 1;
  m_map_position = 1;

  CheckConsistency(nmarkers);

} /* SpacingDS::ctor */

//_____________________________________________________________

SpacingDS::SpacingDS(const vector<long>& positions, long length,
  long offset, long map_position, long nmarkers) :
DataSourceIf(), m_positions(positions), m_length(length), m_offset(offset),
m_map_position(map_position)
{
  std::sort(m_positions.begin(), m_positions.end());

  CheckConsistency(nmarkers);

} /* SpacingDS::maximal ctor */

//_____________________________________________________________

SpacingDS::~SpacingDS()
{
} /* SpacingDS::default dtor */

//_____________________________________________________________

string SpacingDS::getXML(unsigned int numTabs) const
{
  string spaceXML;

  if (m_length == 0) return spaceXML;

  addTabs(numTabs,spaceXML);
  spaceXML += string("<spacing>\n");

  ++numTabs;
  addTabs(numTabs,spaceXML);
  spaceXML += string("<block>\n"); // this will cover loci later, so
                                   // will have a loop here

  ++numTabs;
  addTabs(numTabs,spaceXML);
  spaceXML += string("<map_position> ") + ToString(m_map_position);
  spaceXML += string(" </map_position>\n");

  addTabs(numTabs,spaceXML);
  spaceXML += string("<length> ") + ToString(m_length);
  spaceXML += string(" </length>\n");

  if (!m_positions.empty())
    {
      addTabs(numTabs,spaceXML);
      spaceXML += string("<locations> ");
      vector<long>::const_iterator pos;
      for(pos = m_positions.begin(); pos != m_positions.end(); ++pos)
        {
          spaceXML += ToString(*pos) + " ";
        }
      spaceXML += "\n";
      addTabs(numTabs,spaceXML);
      spaceXML += string("</locations>\n");
    }

  addTabs(numTabs,spaceXML);
  spaceXML += string("<offset> ") + ToString(m_offset);
  spaceXML += string(" </offset>\n");

  --numTabs;
  addTabs(numTabs,spaceXML);
  spaceXML += string("</block>\n");

  --numTabs;
  addTabs(numTabs,spaceXML);
  spaceXML += string("</spacing>\n");

  return spaceXML;

} /* SpacingDS::getXML */

//_____________________________________________________________
