//$Id: ui_vars.cpp,v 1.20 2006/10/25 20:46:46 ewalkup Exp $

/* 
 *  Copyright 2004  Peter Beerli, Mary Kuhner, Jon Yamato and Joseph Felsenstein
 *
 *  This software is distributed free of charge for non-commercial use
 *  and is copyrighted.  Of course, we do not guarantee that the software
 *  works, and are not responsible for any damage you may cause or have.
 *     
 */

#include "datapack.h"
#include "defaults.h"
#include "ui_vars.h"

UIVars::UIVars(DataPack& datapack, string fileName, UIInterface* myui)
    :   ui(myui),
        chains(this), //no prerequisites
        userparams( this,
                    fileName), //no prerequisites
        datapackplus(this,
                     datapack), //no prerequisites
        datamodel(this,  
                  datapack.GetNRegions(),datapack.GetNMaxLoci()),
        //datamodel requires datapackplus
        traitmodels(this),
        //traitmodels also requires datapackplus
        forces(     this,
                    datapack.GetNCrossPartitions(),     
                    datapack.GetNPartitionsByForceType(force_MIG),
                    datapack.GetNPartitionsByForceType(force_DISEASE),
                    datapack.CanMeasureRecombination())
        //forces requires both datapackplus and datamodel.  It might
        // eventually require traitmodels, too!  So.  Definitley last.

        // WARNING!!
        // Here we're loading up the UIVars with information coming from
        // the datapack. However, here in the constructor is the only
        // time we should be doing anything directly from the DataPack.
        // After construction time, access needed to DataPack structures
        // by the UI should go through the UIVarsDataPackPlus class.
        // Ideally, we'd like to refactor this later so that there is
        // and "early" and a "late" version of the DataPack. This restriction
        // on access is the first step in supporting that refactor.
{
}

UIVars::UIVars(const UIVars& vars)
    :   ui(vars.ui),
        chains(this,vars.chains),
        userparams(this,vars.userparams),
        datapackplus(this,vars.datapackplus),
        datamodel(this,vars.datamodel),
        traitmodels(this, vars.traitmodels),
        forces(this,vars.forces)
{
}

UIVars::~UIVars()
{
}

string UIVars::GetParamNameWithConstraint(force_type ftype, long pindex,
                                          bool doLongName) const
{
  //This function has to be here because it uses both forces and datapackplus.

  string shortname = datapackplus.GetParamName(ftype, pindex, false);
  string longname = datapackplus.GetParamName(ftype, pindex, true);
  long gindex = forces.ParamInGroup(ftype, pindex);
  if (gindex != FLAGLONG) {
    LongVec1d gpindexes = forces.GetGroupParamList(ftype, gindex);
    shortname = "";
    longname = "";
    string::size_type commapos = string::npos;
    //This is guaranteed to be overwritten.
    for (unsigned long gpnum=0; gpnum<gpindexes.size(); gpnum++){
      commapos = longname.rfind(",");
      shortname += datapackplus.GetParamName(ftype, gpindexes[gpnum], false)
        + "/";
      longname +=  datapackplus.GetParamName(ftype, gpindexes[gpnum], true)
        + ", ";
    }
    shortname.erase(shortname.size()-1,1);
    longname.erase(longname.size()-2,2);
    if (gpindexes.size()==2) {
      longname.replace(commapos,1," and");
      longname.append(" (both constrained to ");
    }
    else if (gpindexes.size() > 2) {
      longname.insert(commapos+1," and");
      longname.append(" (all constrained to ");
    }
    else {
      assert (gpindexes.size() == 1);
      longname.append(" (the only current member of a potential group, which would be constrained to ");
    }
  }

  switch(forces.GetParamstatus(ftype, pindex)) {
  case pstat_unconstrained:
    if (gindex != FLAGLONG) {
      assert(false); //Groups should not be able to be set 'unconstrained'
      longname.append("start at the same value, then vary freely)");
    }
    else {
      //longname.append(" (unconstrained)");
      // That took too much space, but we could add it back if needed.
    }
    break;
  case pstat_constant:
    if (gindex != FLAGLONG) {
      longname.append("stay at the same constant value)");
    }
    else {
      longname.append(" (held constant)");
    }
    break;
  case pstat_invalid:
    if (gindex != FLAGLONG) {
      longname.append("be held at zero and ignored in output)");
    }
    else {
      longname.append(" (held at zero and ignored in output)");
    }
    break;
  case pstat_joint:
  case pstat_identical:
    longname.append("be identical)");
    break;
  }

  if (doLongName) return longname;
  return shortname;
}
