//$Id: parsetreetodata.h,v 1.12 2007/05/03 20:10:15 lpsmith Exp $

#ifndef PARSETREETODATA_H
#define PARSETREETODATA_H

/* 
 Copyright 2002  Peter Beerli, Mary Kuhner, Jon Yamato and Joseph Felsenstein

 This software is distributed free of charge for non-commercial use
 and is copyrighted.  Of course, we do not guarantee that the software
 works, and are not responsible for any damage you may cause or have.
*/

#include <string>
#include "individual.h"     // for Individual
#include "tinyxml.h"
#include "locus.h"          // for TipData
#include "parsetreewalker.h"

/******************************************************************
 Walks the TinyXML parse tree generated by XmlParser (xml.h)
 and creates the DataPack.

 Based on a class by Jim Sloan, rewritten by Mary Kuhner, and
 then by Elizabeth Walkup
*********************************************************************/

class DataPack;
class Random;
class Region;
class UserTree;


class ParseTreeToData   : public ParseTreeWalker
{
    private:
        DataPack&               datapack;
        Region*                 m_pCurrRegion;
        long                    m_currpopulation;
        long                    m_currindno;
        Individual              m_currIndividual;
        TipData                 m_tipdata;
        std::auto_ptr<Random>   m_randomNameSource;

        // purposely not implemented, this class is meant to be a singleton
        ParseTreeToData();
        ParseTreeToData(const ParseTreeToData&);
        ParseTreeToData& operator=(const ParseTreeToData&);

    protected:

        // for creating less-likely-to-crash names
        std::string XmlRandomLongAsString();

        // main line data reading routines
        void        DoFormat(TiXmlElement*);
        void        DoData(TiXmlElement*);
        void        DoRegion(TiXmlElement*, long regionId);
        void        DoPopulations(TiXmlElement*);
        void        DoPopulation(TiXmlElement*);
        void        DoIndividual(TiXmlElement*);
        void        DoSamples(TiXmlElement *);
        void        DoStatus(TiXmlElement *);
        void        DoDisease(TiXmlElement *);
        void        DoDataBlocks(TiXmlElement *);
        void        DoGenotypeResolutions(TiXmlElement *);

        // auxillary map location information
        void        DoSpacing(TiXmlElement* );
        void        DoBlocks(TiXmlElement* );
        void        DoMapPosition(TiXmlElement* , long locus);
        void        DoLength(TiXmlElement* , long locus);
        void        DoLocations(TiXmlElement* , long locus);
        void        DoOffset(TiXmlElement* , long locus);

        // auxiliary data reading routines
        void        DoPhases(TiXmlElement* );  // DoSamples *must* be called first!
        void        DoEffectivePopSize(TiXmlElement* );

        // main line tree reading routines--currently called in DoRegions()
        UserTree*   DoTree(TiXmlElement* xmlllist);

    public:
        ParseTreeToData(XmlParser&,DataPack&);
        virtual ~ParseTreeToData();
        void        ProcessFileData();        // processes datapack part of structures

};

LongVec1d::iterator FindValIn(long, LongVec1d&);

#endif /* PARSETREETODATA_H */
