//$Id: parsetreetosettings.h,v 1.14 2006/04/25 17:58:56 lpsmith Exp $

#ifndef PARSETREETOSETTINGS_H
#define PARSETREETOSETTINGS_H

/* 
 Copyright 2002  Peter Beerli, Mary Kuhner, Jon Yamato and Joseph Felsenstein

 This software is distributed free of charge for non-commercial use
 and is copyrighted.  Of course, we do not guarantee that the software
 works, and are not responsible for any damage you may cause or have.
*/

#include <string>
#include "defaults.h"       // for force_type
#include "parsetreewalker.h"
#include "vectorx.h"

class TiXmlElement;
class UIInterface;
class UIId;

/******************************************************************
 Walks the TinyXML parse tree generated by XmlParser (xml.h)
 and fills the UIVars variable store via UIInterface

 Based on a class by Jim Sloan, rewritten by Mary Kuhner, and
 then by Elizabeth Walkup
*********************************************************************/


class ParseTreeToSettings   : public ParseTreeWalker
{
    private:
        UIInterface&            uiInterface;

        // purposely not implemented, this class is meant to be a singleton
        ParseTreeToSettings();
        ParseTreeToSettings(const ParseTreeToSettings&);
        ParseTreeToSettings& operator=(const ParseTreeToSettings&);

    protected:

        // for stuffing data into UIInterface
        void DoOptionalElement(TiXmlElement* ancestor, std::string xmlTag, std::string uiIdentifier, UIId id = NO_ID());
        void DoRequiredElement(TiXmlElement* ancestor, std::string xmlTag, std::string uiIdentifier, UIId id = NO_ID());

        // main line data-model reading routines
        void        DoDataModels(TiXmlElement*);
        void        DoRegionDataModels(TiXmlElement*,long regionId);
        void        DoDLModel(TiXmlElement*, long regionId ,long locusId=0);
        void        DoBaseFrequencies(TiXmlElement*, UIId id);
        void        DoGTR(TiXmlElement*, UIId id);
        void        DoCategories(TiXmlElement*,UIId id);

        // main line chain parameter reading routines
        void        DoChainParams(TiXmlElement *);
        void        DoTemperatures(StringVec1d);
        void        DoSwapInterval(TiXmlElement *);

        // main line user parameter reading routines
        void        DoUserParams(TiXmlElement *);

        // main line force parameter reading routines
        void        DoForces(TiXmlElement *);
        void        DoForceIfPresent(TiXmlElement *, force_type forcetype, const string& forcetag);
        void        DoForce(TiXmlElement *, force_type);
        void        DoStartValuesAndMethods(TiXmlElement *, UIId);
	void        DoTrueValues(TiXmlElement *, UIId);
        void        DoProfiles(TiXmlElement *, UIId);
        void        DoConstraints(TiXmlElement *, UIId);
        void        DoGroups(TiXmlElement *, UIId);
        void        DoGroup(TiXmlElement *, UIId, size_t index);
        void        DoPriors(TiXmlElement *, UIId);
        void        DoPrior(TiXmlElement *, UIId);

  void DoTraits(TiXmlElement * dataElement);
  void DoRegionTraits(TiXmlElement * traitsElement, long);
  void DoRegionTrait(TiXmlElement * traitElement, long);
  void DoLocations(TiXmlElement* locationsElement, long, long);
  void DoAnalysis(TiXmlElement* analysisElement, long, long);
  void DoPhenotypes(TiXmlElement* phenotypesElement, long, long);


    public:
        ParseTreeToSettings(XmlParser&, UIInterface &);
        virtual ~ParseTreeToSettings();
        void        ProcessFileSettings();    // processes remaining structures

};

#endif /* PARSETREETOSETTINGS_H */
